/***************************************************************************
 * chord.h: implementation of ChordSelector class
 *
 * This file is part of KTabEdit, a KDE tabulature editor
 *
 * copyright (C) 2003-2006 the KTabEdit development team <mrgwen@gmail.com>
 * copyright (C) 2002-2003 the KGuitar development team <mrgwen@gmail.com>
 ***************************************************************************/

/***************************************************************************
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * See the file COPYING for more information.
 ***************************************************************************/

#ifndef CHORD_H
#define CHORD_H

#include <QDialog>
#include <tse3/MidiScheduler.h>

class QComboBox;
class QLabel;
class QLineEdit;
class QListWidget;
class QPushButton;
class QRadioButton;
class QVBoxLayout;

class ChordList;
class Fingering;
class FingerList;
class Fingers;
class Strumming;
class TabTrack;

class ChordSelector : public QDialog
{
    Q_OBJECT
public:
	ChordSelector(TSE3::MidiScheduler* midiScheduler, const TabTrack& track);
	virtual ~ChordSelector();
	
	virtual void initTonicNoteUI();
	virtual void initBassNoteUI();
	virtual void initChordFunctionUI();
	virtual void initChordExtentionUI();
	virtual void initChordNotesUI();
	virtual void initInverseUI();
	virtual void initComplexityUI();
	virtual void initFingeringUI();
	virtual void initFingerListUI();
	virtual void initButtonUI();
	virtual void initLayoutUI();
	/**
	 * Return the Fingering class.
	 * @return the fingering
	 */
	Fingering & getFingering() const;

	/**
	 * Return the chord list.
	 * @return the chord list
 	 */
	ChordList & getChordList() const;

	/**
	 * Return the fret number at the string x.
	 * @param x the string number
	 * @result the fret
	 */
	int app(int x);
	
public slots:
	/**
		Try to detect some chord forms from a given applicature.
	*/
	void detectChord();
	
private slots:
	void setStep3();
	void setHighSteps();
	void setStepsFromChord();
	
	/**
		Analyses st[] combobox array and find out steps templating
		listboxes selections from it
	*/
	void findSelection();
	
	/**
		Most complex and longest method that does the real calculation of
		chords. Translates steps settings input to the list of chords and
		adds them to displayer.
	*/
	void findChords();
	
	/**
		Pops out the strumming dialog.
		// TODO What settings does that really change?
	*/
	void askStrum();
	
	/**
		Try to decipher text-written chord name and set steps accordingly
	*/
	void analyzeChordName();
	
	/**
		Analyze chord by text-written name, automatically select best one
		and insert it into track.
	*/
	void quickInsert();
	
	/**
		Play a chord
	*/
	void playMidi();
	
private:
	// Chord definition
	//
	static const int msTemplate[][6];
	
	/*
		Calculate absolute notes from tonic + step user input
	*/
	bool calculateNotesFromSteps(int[], int &);
	void setApp(int x, int fret);
	int  scheme();
	
	const TabTrack& mTabTrack;
	TSE3::MidiScheduler* mScheduler;
	Fingering* mFingerging;
	FingerList* mFingerList;
	ChordList* mChords;
	
	QLineEdit* mChordName;
	QListWidget* mTonic;
	QListWidget* mStep3;
	QListWidget* mStepHigh;
	QComboBox* mSt[7];
	QComboBox* mInv;
	QComboBox* mBassNote;
	QLabel* mCnote[7];
	QLabel* stlabel[7];
	QVBoxLayout* mComplexity;
	QRadioButton* mComplexer[3];
	QPushButton* mPlay;
	QPushButton* ok;
	QPushButton* cancel;
	QPushButton* strumbut;
	QPushButton* chordNameAnalyze;
	QPushButton* chordNameQuickInsert;
	
	int mStrumScheme;
};

#endif
