/***************************************************************************
 * chordlistitem.h: implementation of ChordListItem class
 *
 * This file is part of KTabEdit, a KDE tabulature editor
 *
 * copyright (C) 2003-2006 the KTabEdit development team <mrgwen@gmail.com>
 * copyright (C) 2002-2003 the KGuitar development team <mrgwen@gmail.com>
 ***************************************************************************/

/***************************************************************************
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * See the file COPYING for more information.
 ***************************************************************************/

#ifndef CHORDLISTITEM_H
#define CHORDLISTITEM_H

#include <QListWidgetItem>

/** 
	The ChordListItem class. This class builds and displays chord list name.
*/
class ChordListItem : public QListWidgetItem {
public:
	/**
		Construct the chord list.
		The constructor builds the chord name with is tonic, bass, third, ... frets.
		\param tonic the tonic fretted
		\parama s3 the third fretted
		\param s7 the seventh fretted
		\param s9 the ninth fretted
		\param s11 the eleventh fretted
		\param s13 the thirtheenth fretted
	*/
	ChordListItem(int tonic, int s3, int s5, int s7, int s9, int s11, int s13);
	
	~ChordListItem();

	/**
		Get the tonic.
		\return the tonic
	*/
	int tonic() const;
	
	/**
		Get the step information.
		\return the step information
	*/
	int step(int x) const;

private:
	static const int msToneShift[6];
	int mTonic;
	int mStep[6];
};

#endif
