/***************************************************************************
 * optionsprinting.h: implementation of OptionsPrinting class
 *
 * This file is part of KTabEdit, a KDE tabulature editor
 *
 * copyright (C) 2003-2006 the KTabEdit development team <mrgwen@gmail.com>
 * copyright (C) 2002-2003 the KGuitar development team <mrgwen@gmail.com>
 ***************************************************************************/

/***************************************************************************
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * See the file COPYING for more information.
 ***************************************************************************/

#ifndef OPTIONSPRINTING_H
#define OPTIONSPRINTING_H

#include <KConfig>
#include <QWidget>
#include <QVBoxLayout>
#include <QRadioButton>

#include "optionspage.h"

/**
	The OptionsPrinting class. This class creates a widget inside this widget
	the user can configure the printing options.
*/
class OptionsPrinting: public OptionsPage {
	Q_OBJECT
public:
	/**
		Construct the printing widget.
		The constructor creates the widget and inits the parent (optionPage). Here are
		the printing options :
			tablature or notes.
		@param config config is used to read and write the KTabEdit config file.
		@param parent the widget parent
		@param name the widget name
	*/
	OptionsPrinting(KConfig* config, QWidget* parent);
	
	/**
		Called when the user click on the Apply button.
		The printing configuration is written on the KTabEdit config file.
	*/
	virtual void applyBtnClicked();
	
	/**
		Called when the user click on the Default button.
		It does nothing.
	*/
	virtual void defaultBtnClicked();
	
private:
	QVBoxLayout* mStyleGroup;
	QRadioButton* mStyle[4];
};

#endif
